package xyz.ibatv.nethercraft.block.entity;

// REMOVE "BARRELS" FOR ACTUAL CHESTS AT SOME POINT.

import xyz.ibatv.nethercraft.block.MoreBarrelBlock;
import xyz.ibatv.nethercraft.init.MblvBlockInit;
import net.minecraft.block.BarrelBlock;
import net.minecraft.block.BlockState;
import net.minecraft.block.entity.LootableContainerBlockEntity;
import net.minecraft.block.entity.ViewerCountManager;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.entity.player.PlayerInventory;
import net.minecraft.inventory.Inventories;
import net.minecraft.inventory.Inventory;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NbtCompound;
import net.minecraft.registry.RegistryWrapper;
import net.minecraft.screen.GenericContainerScreenHandler;
import net.minecraft.screen.ScreenHandler;
import net.minecraft.sound.SoundCategory;
import net.minecraft.sound.SoundEvent;
import net.minecraft.sound.SoundEvents;
import net.minecraft.text.Text;
import net.minecraft.util.collection.DefaultedList;
import net.minecraft.util.math.BlockPos;
import net.minecraft.util.math.Direction;
import net.minecraft.util.math.Vec3i;
import net.minecraft.world.World;
import org.jetbrains.annotations.NotNull;

public class MoreBarrelBlockEntity extends LootableContainerBlockEntity implements MoreBarrelBlockEntityInterface {
    private DefaultedList<ItemStack> items;
    private final ViewerCountManager openersCounter;

    public MoreBarrelBlockEntity(BlockPos blockPos, BlockState blockState) {
        super(MblvBlockInit.MORE_BARREL_BLOCK_ENTITY, blockPos, blockState);
        this.items = DefaultedList.ofSize(27, ItemStack.EMPTY);
        this.openersCounter = new ViewerCountManager() {
            protected void onContainerOpen(World level, BlockPos pos, BlockState state) {
                MoreBarrelBlockEntity.this.playSound(state, SoundEvents.BLOCK_BARREL_OPEN);
                MoreBarrelBlockEntity.this.updateBlockState(state, true);
            }

            protected void onContainerClose(World level, BlockPos pos, BlockState state) {
                MoreBarrelBlockEntity.this.playSound(state, SoundEvents.BLOCK_BARREL_CLOSE);
                MoreBarrelBlockEntity.this.updateBlockState(state, false);
            }

            protected void onViewerCountUpdate(World level, BlockPos pos, BlockState state, int count, int openCount) {
            }

            protected boolean isPlayerViewing(PlayerEntity player) {
                if (player.currentScreenHandler instanceof GenericContainerScreenHandler) {
                    Inventory container = ((GenericContainerScreenHandler)player.currentScreenHandler).getInventory();
                    return container == MoreBarrelBlockEntity.this;
                } else {
                    return false;
                }
            }
        };
    }

    @Override
    protected void writeNbt(NbtCompound tag, RegistryWrapper.WrapperLookup registries) {
        super.writeNbt(tag, registries);
        if (!this.writeLootTable(tag)) {
            Inventories.writeNbt(tag, this.items, registries);
        }

    }

    @Override
    protected void readNbt(NbtCompound tag, RegistryWrapper.WrapperLookup registries) {
        super.readNbt(tag, registries);
        this.items = DefaultedList.ofSize(this.size(), ItemStack.EMPTY);
        if (!this.readLootTable(tag)) {
            Inventories.readNbt(tag, this.items, registries);
        }

    }

    @Override
    public int size() {
        return 27;
    }

    @Override
    protected @NotNull DefaultedList<ItemStack> getHeldStacks() {
        return this.items;
    }

    @Override
    protected void setHeldStacks(DefaultedList<ItemStack> items) {
        this.items = items;
    }

    @Override
    protected @NotNull Text getContainerName() {
        return Text.translatable("container.nethercraft-classic." + getBlock().barrelWoodType + "_barrel");
    }

    @Override
    protected @NotNull ScreenHandler createScreenHandler(int containerId, PlayerInventory inventory) {
        return GenericContainerScreenHandler.createGeneric9x3(containerId, inventory, this);
    }

    @Override
    public void onOpen(PlayerEntity player) {
        if (!this.removed && !player.isSpectator()) {
            this.openersCounter.openContainer(player, this.getWorld(), this.getPos(), this.getCachedState());
        }

    }

    @Override
    public void onClose(PlayerEntity player) {
        if (!this.removed && !player.isSpectator()) {
            this.openersCounter.closeContainer(player, this.getWorld(), this.getPos(), this.getCachedState());
        }

    }

    @Override
    public void recheckOpen() {
        if (!this.removed) {
            this.openersCounter.updateViewerCount(this.getWorld(), this.getPos(), this.getCachedState());
        }

    }

    @Override
    public void updateBlockState(BlockState state, boolean open) {
        this.world.setBlockState(this.getPos(), (BlockState)state.with(BarrelBlock.OPEN, open), 3);
    }

    @Override
    public void playSound(BlockState state, SoundEvent sound) {
        Vec3i vec3i = ((Direction)state.get(BarrelBlock.FACING)).getVector();
        double d = (double)this.pos.getX() + 0.5 + (double)vec3i.getX() / 2.0;
        double e = (double)this.pos.getY() + 0.5 + (double)vec3i.getY() / 2.0;
        double f = (double)this.pos.getZ() + 0.5 + (double)vec3i.getZ() / 2.0;
        assert this.world != null;
        this.world.playSound((PlayerEntity)null, d, e, f, sound, SoundCategory.BLOCKS, 0.5F, this.world.random.nextFloat() * 0.1F + 0.9F);
    }

    public MoreBarrelBlock getBlock() {
        return (MoreBarrelBlock) getCachedState().getBlock();
    }
}