package xyz.ibatv.nethercraft.world;

import net.minecraft.block.Blocks;
import net.minecraft.registry.Registerable;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.registry.tag.BlockTags;
import net.minecraft.structure.rule.RuleTest;
import net.minecraft.structure.rule.TagMatchRuleTest;
import net.minecraft.util.Identifier;
import net.minecraft.util.math.intprovider.ConstantIntProvider;
import net.minecraft.world.gen.blockpredicate.BlockPredicate;
import net.minecraft.world.gen.feature.*;
import net.minecraft.world.gen.feature.size.TwoLayersFeatureSize;
import net.minecraft.world.gen.foliage.BlobFoliagePlacer;
import net.minecraft.world.gen.stateprovider.BlockStateProvider;
import net.minecraft.world.gen.trunk.StraightTrunkPlacer;
import xyz.ibatv.nethercraft.NethercraftClassic;
import xyz.ibatv.nethercraft.block.ModBlocks;

import java.util.List;

public class ModConfiguredFeatures {
    public static final RegistryKey<ConfiguredFeature<?, ?>> FOULITE_ORE_KEY = registerKey("foulite_ore");
    public static final RegistryKey<ConfiguredFeature<?, ?>> W_ORE_KEY = registerKey("w_ore");
    public static final RegistryKey<ConfiguredFeature<?, ?>> LINIUM_ORE_KEY = registerKey("linium_ore");
    public static final RegistryKey<ConfiguredFeature<?, ?>> NERIDIUM_ORE_KEY = registerKey("neridium_ore");
    public static final RegistryKey<ConfiguredFeature<?, ?>> PYRIDIUM_ORE_KEY = registerKey("pyridium_ore");

    public static final RegistryKey<ConfiguredFeature<?, ?>> GLOWOOD_KEY = registerKey("glowood");



    public static final RegistryKey<ConfiguredFeature<?, ?>> CORRUPTED_TREE = registerKey("corrupted_tree");
    public static final RegistryKey<ConfiguredFeature<?, ?>> CORRUPTED_TREE_PLANTED = registerKey("corrupted_tree_planted");


    public static void bootstrap(Registerable<ConfiguredFeature<?,?>> context) {

        RuleTest netherReplaceables = new TagMatchRuleTest(BlockTags.BASE_STONE_NETHER);

        List<OreFeatureConfig.Target> netherFouliteOres =
                List.of(OreFeatureConfig.createTarget(netherReplaceables, ModBlocks.FOULITE_ORE.getDefaultState()));
        register(context, FOULITE_ORE_KEY, Feature.ORE, new OreFeatureConfig(netherFouliteOres, 12));

        List<OreFeatureConfig.Target> netherWOres =
                List.of(OreFeatureConfig.createTarget(netherReplaceables, ModBlocks.W_ORE.getDefaultState()));
        register(context, W_ORE_KEY, Feature.ORE, new OreFeatureConfig(netherWOres, 12));

        List<OreFeatureConfig.Target> netherLiniumOres =
                List.of(OreFeatureConfig.createTarget(netherReplaceables, ModBlocks.LINIUM_ORE.getDefaultState()));
        register(context, LINIUM_ORE_KEY, Feature.ORE, new OreFeatureConfig(netherLiniumOres, 12));

        List<OreFeatureConfig.Target> netherNeridiumOres =
                List.of(OreFeatureConfig.createTarget(netherReplaceables, ModBlocks.NERIDIUM_ORE.getDefaultState()));
        register(context, NERIDIUM_ORE_KEY, Feature.ORE, new OreFeatureConfig(netherNeridiumOres, 12));

        List<OreFeatureConfig.Target> netherPyridiumOres =
                List.of(OreFeatureConfig.createTarget(netherReplaceables, ModBlocks.PYRIDIUM_ORE.getDefaultState()));
        register(context, PYRIDIUM_ORE_KEY, Feature.ORE, new OreFeatureConfig(netherPyridiumOres, 12));


        register(context, GLOWOOD_KEY, Feature.TREE, new TreeFeatureConfig.Builder(
                BlockStateProvider.of(ModBlocks.GLOWOOD_LOG),
                new StraightTrunkPlacer(4,2,0),

                BlockStateProvider.of(ModBlocks.GLOWOOD_LEAVES),
                new BlobFoliagePlacer(ConstantIntProvider.create(2), ConstantIntProvider.create(0), 3),

                new TwoLayersFeatureSize(1, 0, 1)).dirtProvider(BlockStateProvider.of(Blocks.NETHERRACK)).build());

        BlockPredicate blockPredicate = BlockPredicate.matchingBlocks(
        );

        register(context, CORRUPTED_TREE, Feature.HUGE_FUNGUS, new HugeFungusFeatureConfig(
                ModBlocks.NETHER_DIRT.getDefaultState(),
                ModBlocks.NETHER_DIRT.getDefaultState(),
                ModBlocks.NETHER_DIRT.getDefaultState(),
                ModBlocks.NETHER_DIRT.getDefaultState(),
                blockPredicate,
                false
        ));

        register(context, CORRUPTED_TREE_PLANTED, Feature.HUGE_FUNGUS, new HugeFungusFeatureConfig(
                ModBlocks.NETHER_DIRT.getDefaultState(),
                ModBlocks.NETHER_DIRT.getDefaultState(),
                ModBlocks.NETHER_DIRT.getDefaultState(),
                ModBlocks.NETHER_DIRT.getDefaultState(),
                blockPredicate,
                true
        ));

    }



    public static RegistryKey<ConfiguredFeature<?, ?>> registerKey(String name) {
        return RegistryKey.of(RegistryKeys.CONFIGURED_FEATURE, Identifier.of(NethercraftClassic.MOD_ID, name));
    }

    private static <FC extends FeatureConfig, F extends Feature<FC>> void register(Registerable<ConfiguredFeature<?, ?>> context,
                                                                                   RegistryKey<ConfiguredFeature<?, ?>> key, F feature, FC configuration) {
        context.register(key, new ConfiguredFeature<>(feature, configuration));
    }
}