package xyz.ibatv.nethercraft.world;

import net.minecraft.registry.Registerable;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.registry.entry.RegistryEntry;
import net.minecraft.util.Identifier;
import net.minecraft.world.gen.YOffset;
import net.minecraft.world.gen.blockpredicate.BlockPredicate;
import net.minecraft.world.gen.feature.*;
import net.minecraft.world.gen.placementmodifier.BlockFilterPlacementModifier;
import net.minecraft.world.gen.placementmodifier.CountMultilayerPlacementModifier;
import net.minecraft.world.gen.placementmodifier.HeightRangePlacementModifier;
import net.minecraft.world.gen.placementmodifier.PlacementModifier;
import xyz.ibatv.nethercraft.NethercraftClassic;
import xyz.ibatv.nethercraft.block.ModBlocks;

import java.util.List;

public class ModPlacedFeatures {
    public static final RegistryKey<PlacedFeature> FOULITE_ORE_PLACED_KEY = registerKey("foulite_ore_placed");
    public static final RegistryKey<PlacedFeature> W_ORE_PLACED_KEY = registerKey("w_ore_placed");
    public static final RegistryKey<PlacedFeature> LINIUM_ORE_PLACED_KEY = registerKey("linium_ore_placed");
    public static final RegistryKey<PlacedFeature> NERIDIUM_ORE_PLACED_KEY = registerKey("neridium_ore_placed");
    public static final RegistryKey<PlacedFeature> PYRIDIUM_ORE_PLACED_KEY = registerKey("pyridium_ore_placed");

    public static final RegistryKey<PlacedFeature> GLOWOOD_PLACED_KEY = registerKey("glowood_placed");


    public static final RegistryKey<PlacedFeature> PLACED_CORRUPTED_TREE = registerKey("corrupted_tree");
    public static final RegistryKey<PlacedFeature> PLACED_CORRUPTED_TREE_PLANTED = registerKey("corrupted_tree_planted");

    public static void bootstrap(Registerable<PlacedFeature> context) {
        var configuredFeatures = context.getRegistryLookup(RegistryKeys.CONFIGURED_FEATURE);

        register(context, FOULITE_ORE_PLACED_KEY, configuredFeatures.getOrThrow(ModConfiguredFeatures.FOULITE_ORE_KEY),
                ModOrePlacement.modifiersWithCount(14,
                        HeightRangePlacementModifier.uniform(YOffset.fixed(16), YOffset.fixed(110))));

        register(context, W_ORE_PLACED_KEY, configuredFeatures.getOrThrow(ModConfiguredFeatures.W_ORE_KEY),
                ModOrePlacement.modifiersWithCount(14,
                        HeightRangePlacementModifier.uniform(YOffset.fixed(16), YOffset.fixed(110))));

        register(context, LINIUM_ORE_PLACED_KEY, configuredFeatures.getOrThrow(ModConfiguredFeatures.LINIUM_ORE_KEY),
                ModOrePlacement.modifiersWithCount(14,
                        HeightRangePlacementModifier.uniform(YOffset.fixed(16), YOffset.fixed(110))));

        register(context, NERIDIUM_ORE_PLACED_KEY, configuredFeatures.getOrThrow(ModConfiguredFeatures.NERIDIUM_ORE_KEY),
                ModOrePlacement.modifiersWithCount(14,
                        HeightRangePlacementModifier.uniform(YOffset.fixed(16), YOffset.fixed(110))));

        register(context, PYRIDIUM_ORE_PLACED_KEY, configuredFeatures.getOrThrow(ModConfiguredFeatures.PYRIDIUM_ORE_KEY),
                ModOrePlacement.modifiersWithCount(14,
                        HeightRangePlacementModifier.uniform(YOffset.fixed(16), YOffset.fixed(110))));


        /*register(context, GLOWOOD_PLACED_KEY, configuredFeatures.getOrThrow(ModConfiguredFeatures.GLOWOOD_KEY),
                VegetationPlacedFeatures.treeModifiersWithWouldSurvive(
                        PlacedFeatures.createCountExtraModifier(2, 0.1f, 2), ModBlocks.GLOWOOD_SAPLING));*/


        register(context, GLOWOOD_PLACED_KEY, configuredFeatures.getOrThrow(ModConfiguredFeatures.GLOWOOD_KEY),
                List.of(CountMultilayerPlacementModifier.of(4)));

        //register(context, GLOWOOD_PLACED_KEY, configuredFeatures.getOrThrow(ModConfiguredFeatures.GLOWOOD_KEY),
        //        VegetationPlacedFeatures.treeModifiersWithWouldSurvive(PlacedFeatures.createCountExtraModifier(2,0.1f,2), ModBlocks.GLOWOOD_SAPLING));

        register(context, PLACED_CORRUPTED_TREE, configuredFeatures.getOrThrow(ModConfiguredFeatures.CORRUPTED_TREE),
                List.of(CountMultilayerPlacementModifier.of(4)));

        register(context, PLACED_CORRUPTED_TREE_PLANTED, configuredFeatures.getOrThrow(ModConfiguredFeatures.CORRUPTED_TREE_PLANTED),
                List.of(BlockFilterPlacementModifier.of(BlockPredicate.IS_AIR)));

    }

    public static RegistryKey<PlacedFeature> registerKey(String name) {
        return RegistryKey.of(RegistryKeys.PLACED_FEATURE, Identifier.of(NethercraftClassic.MOD_ID, name));
    }

    private static void register(Registerable<PlacedFeature> context, RegistryKey<PlacedFeature> key, RegistryEntry<ConfiguredFeature<?, ?>> configuration,
                                 List<PlacementModifier> modifiers) {
        context.register(key, new PlacedFeature(configuration, List.copyOf(modifiers)));
    }

    private static <FC extends FeatureConfig, F extends Feature<FC>> void register(Registerable<PlacedFeature> context, RegistryKey<PlacedFeature> key,
                                                                                   RegistryEntry<ConfiguredFeature<?, ?>> configuration,
                                                                                   PlacementModifier... modifiers) {
        register(context, key, configuration, List.of(modifiers));
    }
}
